PounceCam = LibStub("AceAddon-3.0"):NewAddon("PounceCam", "AceEvent-3.0", "AceTimer-3.0")

--==== upvalues ====--
local _G, PounceCam, LibStub, format, select = _G, PounceCam, LibStub, format, select
local GetCVar, SetCVar, GetSpellInfo, UnitClass, UnitGUID = GetCVar, SetCVar, GetSpellInfo, UnitClass, UnitGUID
--==================--


local CEvent = { module = PounceCam }
CEvent.__index = CEvent
function CEvent:Release()
	if self.handler then 
		self.module:UnregisterEvent(self.eventName)
		self.handler = nil
	end
end
function CEvent:Set(handler)
	if handler == self.handler then return end
	self:Release()
	self.module:RegisterEvent(self.eventName, handler)
	self.handler = handler
end
local Event_CLEU = setmetatable( { eventName = "COMBAT_LOG_EVENT_UNFILTERED" }, CEvent )

local function NameWithForm(id)
	local name, rank = GetSpellInfo(id)
	return format("%s (%s)", name, rank)
end

local strPlayerGUID
local strCameraFollowStyle
local strCameraFollowSpeed

local idSpellCatCharge = 49376
local idSpellCatSkullBash = 80965
local idSpellBearCharge = 16979
local idSpellBearSkullBash = 80964
-- local idSpellRogueShadowstep = 36554
-- local idSpellRogueKillingSpree = 51690
local idSpellWarriorCharge = 100
local idSpellWarriorIntercept = 20252
local idSpellWarriorIntervene = 3411

local tblOptionsDefaults = {
	global = {
		CameraResetDelay            = 1,
		[idSpellBearCharge]         = true,
		[idSpellBearSkullBash]      = false,
		[idSpellCatCharge]          = true,
		[idSpellCatSkullBash]       = false,
--		[idSpellRogueShadowstep]    = true,
--		[idSpellRogueKillingSpree]  = true,
		[idSpellWarriorCharge]      = true,
		[idSpellWarriorIntercept]   = true,
		[idSpellWarriorIntervene]   = true,
	}
}

local tblMain = {
	{ strClass = "DRUID",   icon = false, funcGetName = NameWithForm, idSpell = idSpellBearCharge         },
	{ strClass = "DRUID",   icon = true,  funcGetName = NameWithForm, idSpell = idSpellCatCharge          },
	{ strClass = "DRUID",   icon = false, funcGetName = NameWithForm, idSpell = idSpellBearSkullBash      },
	{ strClass = "DRUID",   icon = true,  funcGetName = NameWithForm, idSpell = idSpellCatSkullBash       },

--	{ strClass = "ROGUE",   icon = true,  funcGetName = GetSpellInfo, idSpell = idSpellRogueShadowstep    },
--	{ strClass = "ROGUE",   icon = true,  funcGetName = GetSpellInfo, idSpell = idSpellRogueKillingSpree  },

	{ strClass = "WARRIOR", icon = true,  funcGetName = GetSpellInfo, idSpell = idSpellWarriorCharge      },
	{ strClass = "WARRIOR", icon = true,  funcGetName = GetSpellInfo, idSpell = idSpellWarriorIntercept   },
	{ strClass = "WARRIOR", icon = true,  funcGetName = GetSpellInfo, idSpell = idSpellWarriorIntervene   },
}

local function CheckHandler(strClass)
	local db = PounceCam.db.global

	for _,v in ipairs(tblMain) do
		if (v.strClass == strClass and db[v.idSpell]) then
			Event_CLEU:Set("CLEU")
			return
		end
	end

	Event_CLEU:Release()
end


-- Called when the addon is loaded
function PounceCam:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("PounceCamDB", tblOptionsDefaults)
end

-- Called when the addon is enabled
function PounceCam:OnEnable()
	local _,class = UnitClass("player")
	for _,v in pairs(tblMain) do
		if (class == v.strClass) then
			LibStub("AceConfig-3.0"):RegisterOptionsTable("PounceCam", PounceCam:GetOptionsUI(class))
			self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("PounceCam")

			strPlayerGUID = UnitGUID("player")
			CheckHandler(class)
			return
		end
	end
end

-- Called when the addon is Disabled
function PounceCam:OnDisable()
	Event_CLEU:Release()
end


function PounceCam:ResetCameraFollow()
	SetCVar("cameraSmoothStyle",    strCameraFollowStyle)
	SetCVar("cameraYawSmoothSpeed", strCameraFollowSpeed)
	-- print("resetting camera")
end

function PounceCam:CLEU(_, ...)
	local eventType = select(2, ...)
	local sourceGUID = select(4, ...)

	if(sourceGUID == strPlayerGUID and eventType == "SPELL_CAST_SUCCESS") then
		local spellId = select(12, ...)

		if (self.db.global[spellId]) then
			strCameraFollowStyle = GetCVar("cameraSmoothStyle")
			strCameraFollowSpeed = GetCVar("cameraYawSmoothSpeed")

			SetCVar("cameraSmoothStyle",    "3")
			SetCVar("cameraYawSmoothSpeed", "270")
			PounceCam:ScheduleTimer("ResetCameraFollow", self.db.global.CameraResetDelay)
		end	
	end
end

function PounceCam:GetOptionsUI(strClass)
	local tblArgs = {}
	local index = 0

	tblArgs["TurninatorResetDelaySlider"] = {
		order = index,
		type = "range",
		name = "Camera Reset Delay",
		desc = "amount of time to wait before resetting camera follow style back to user preference",
		min = 0,
		softMin = .5,
		softMax = 3,
		get = function(info) return self.db.global.CameraResetDelay end,
		set = function(info,value) self.db.global.CameraResetDelay = value end,
	}

	for _,v in ipairs(tblMain) do
		if (v.strClass == strClass) then
			index = index + 1
			tblArgs["TurninatorCheckBox" .. index] = {
				order = index,
				width = "full",
				type = "toggle",
				name = v.funcGetName(v.idSpell),
				image = select(3, GetSpellInfo(v.idSpell)),
				get = function(info) return PounceCam.db.global[v.idSpell] end,
				set = function(info, value) 
					PounceCam.db.global[v.idSpell] = value 
					CheckHandler(v.strClass)
				end,
			}
		end
	end

	return {
		name = "PounceCam",
		handler = PounceCam,
		type = 'group',	
		childGroups = "tree",
		args = tblArgs,
		}
end

